@@ -108,8 +108,7 @@ fs_destroy(void *driverarg, void *dbdata);
  */
 
 static isc_boolean_t
-is_safe(const char *input)
-{
+is_safe(const char *input) {
 	unsigned int i;
 	unsigned int len = strlen(input);
 
@@ -119,13 +118,13 @@ is_safe(const char *input)
 		if (input[i] == '.') {
 			/* '.' is not allowed as first char */
 			if (i == 0)
-				return ISC_FALSE;
+				return (ISC_FALSE);
 			/* '..', two dots together is not allowed. */
 			else if (input[i-1] == '.')
-				return ISC_FALSE;
+				return (ISC_FALSE);
 			/* '.' is not allowed as last char */
 			if (i == len)
-				return ISC_FALSE;
+				return (ISC_FALSE);
 			/* only 1 dot in ok location, continue at next char */
 			continue;
 		}
@@ -161,16 +160,14 @@ is_safe(const char *input)
 		 * if we reach this point we have encountered a
 		 * disallowed char!
 		 */
-		return ISC_FALSE;
+		return (ISC_FALSE);
 	}
         /* everything ok. */
-	return ISC_TRUE;
+	return (ISC_TRUE);
 }
 
 static isc_result_t
-create_path_helper(char *out, const char *in, config_data_t *cd)
-{
-
+create_path_helper(char *out, const char *in, config_data_t *cd) {
 	char *tmpString;
 	char *tmpPtr;
 	int i;
@@ -238,6 +235,7 @@ create_path(const char *zone, const char *host, const char *client,
 	int pathsize;
 	int len;
 	isc_result_t result;
+	isc_boolean_t isroot = ISC_FALSE;
 
 	/* we require a zone & cd parameter */
 	REQUIRE(zone != NULL);
@@ -252,16 +250,20 @@ create_path(const char *zone, const char *host, const char *client,
 		 (host != NULL && client == NULL) ||
 		 (host == NULL && client != NULL) );
 
+	/* special case for root zone */
+	if (strcmp(zone, ".") == 0)
+		isroot = ISC_TRUE;
+
 	/* if the requested zone is "unsafe", return error */
-	if (is_safe(zone) != ISC_TRUE)
+	if (!isroot && !is_safe(zone))
 		return (ISC_R_FAILURE);
 
 	/* if host was passed, verify that it is safe */
-	if ((host != NULL) && (is_safe(host) != ISC_TRUE) )
+	if (host != NULL && !is_safe(host))
 		return (ISC_R_FAILURE);
 
-	/* if host was passed, verify that it is safe */
-	if ((client != NULL) && (is_safe(client) != ISC_TRUE) )
+	/* if client was passed, verify that it is safe */
+	if (client != NULL && !is_safe(client))
 		return (ISC_R_FAILURE);
 
 	/* Determine how much memory the split up string will require */
@@ -302,8 +304,11 @@ create_path(const char *zone, const char *host, const char *client,
 	strcpy(tmpPath, cd->basedir);
 
 	/* add zone name - parsed properly */
-	if ((result = create_path_helper(tmpPath, zone, cd)) != ISC_R_SUCCESS)
-		goto cleanup_mem;
+	if (!isroot) {
+		result = create_path_helper(tmpPath, zone, cd);
+		if (result != ISC_R_SUCCESS)
+			goto cleanup_mem;
+	}
 
 	/*
 	 * When neither client or host is passed we are building a
@@ -356,11 +361,11 @@ create_path(const char *zone, const char *host, const char *client,
 		isc_mem_free(ns_g_mctx, tmpPath);
 
 	/* free tmpPath memory */
-	return result;
+	return (result);
 }
 
 static isc_result_t
-process_dir(isc_dir_t dir, void *passback, config_data_t *cd,
+process_dir(isc_dir_t *dir, void *passback, config_data_t *cd,
 	    dlist_t *dir_list, unsigned int basedirlen)
 {
 
@@ -386,10 +391,10 @@ process_dir(isc_dir_t dir, void *passback, config_data_t *cd,
 	foundHost = ISC_FALSE;
 
 	/* copy base directory name to tmp. */
-	strcpy(tmp, dir.dirname);
+	strcpy(tmp, dir->dirname);
 
-	/* dir.dirname will always have '*' as the last char. */
-	astPos = strlen(dir.dirname) - 1;
+	/* dir->dirname will always have '*' as the last char. */
+	astPos = strlen(dir->dirname) - 1;
 
 	/* if dir_list != NULL, were are performing a zone xfr */
 	if (dir_list != NULL) {
@@ -407,56 +412,66 @@ process_dir(isc_dir_t dir, void *passback, config_data_t *cd,
 					 */
 					while ((tmpPtr = strrchr(tmpString,
 								 cd->pathsep))
-					       != NULL) {
+					       != NULL)
+					{
+						if ((strlen(host) +
+						     strlen(tmpPtr + 1) + 2)
+						    > ISC_DIR_NAMEMAX)
+							continue;
 						strcat(host, tmpPtr + 1);
 						strcat(host, ".");
 						tmpPtr[0] = '\0';
 					}
-					strcat(host, tmpString);
+					if ((strlen(host) +
+					     strlen(tmpString) + 1)
+					    <= ISC_DIR_NAMEMAX)
+						strcat(host, tmpString);
 				}
 
 				foundHost = ISC_TRUE;
 				/* set tmp again for use later */
-				strcpy(tmp, dir.dirname);
+				strcpy(tmp, dir->dirname);
 			}
 		} else {
 			/*
 			 * if splitcnt != 0 determine host from
 			 * ".host" directory entry
 			 */
-			while (isc_dir_read(&dir) == ISC_R_SUCCESS) {
+			while (isc_dir_read(dir) == ISC_R_SUCCESS) {
 				if (strncasecmp(".host",
-						dir.entry.name, 5) == 0) {
+						dir->entry.name, 5) == 0) {
 					/*
 					 * handle filesystem's special
 					 * wildcard "-"
 					 */
-					if (strcmp((char *) &dir.entry.name[6],
+					if (strcmp((char *) &dir->entry.name[6],
 						   "-") == 0)
 						strcpy(host, "*");
-					else
-						strcpy(host,
-						       (char *)
-						       &dir.entry.name[6]);
+					else {
+						strncpy(host,
+						   (char *) &dir->entry.name[6],
+						   sizeof(host) - 1);
+						host[255] = '\0';
+					}
 					foundHost = ISC_TRUE;
 					break;
 				}
 			}
 			/* reset dir list for use later */
-			isc_dir_reset(&dir);
+			isc_dir_reset(dir);
 		} /* end of else */
 	}
 
-	while (isc_dir_read(&dir) == ISC_R_SUCCESS) {
+	while (isc_dir_read(dir) == ISC_R_SUCCESS) {
 
 		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
 			      DNS_LOGMODULE_DLZ, ISC_LOG_DEBUG(1),
 			      "Filesystem driver Dir name:"
 			      " '%s' Dir entry: '%s'\n",
-			      dir.dirname, dir.entry.name);
+			      dir->dirname, dir->entry.name);
 
 		/* skip any entries starting with "." */
-		if (dir.entry.name[0] == '.')
+		if (dir->entry.name[0] == '.')
 			continue;
 
 		/*
@@ -469,7 +484,7 @@ process_dir(isc_dir_t dir, void *passback, config_data_t *cd,
 		tmp[astPos] = '\0';
 
 		/* add name to base directory name. */
-		strcat(tmp, dir.entry.name);
+		strcat(tmp, dir->entry.name);
 
 		/* make sure we can stat entry */
 		if (stat(tmp, &sb) == 0 ) {
@@ -507,7 +522,7 @@ process_dir(isc_dir_t dir, void *passback, config_data_t *cd,
 		} else /* if we cannot stat entry, skip it. */
 			continue;
 
-		type = dir.entry.name;
+		type = dir->entry.name;
 		ttlStr = strchr(type,  cd->separator);
 		if (ttlStr == NULL) {
 			isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
@@ -515,7 +530,7 @@ process_dir(isc_dir_t dir, void *passback, config_data_t *cd,
 				      "Filesystem driver: "
 				      "%s could not be parsed properly",
 				      tmp);
-			return ISC_R_FAILURE;
+			return (ISC_R_FAILURE);
 		}
 
 		/* replace separator char with NULL to split string */
@@ -530,7 +545,7 @@ process_dir(isc_dir_t dir, void *passback, config_data_t *cd,
 				      "Filesystem driver: "
 				      "%s could not be parsed properly",
 				      tmp);
-			return ISC_R_FAILURE;
+			return (ISC_R_FAILURE);
 		}
 
 		/* replace separator char with NULL to split string */
@@ -568,10 +583,10 @@ process_dir(isc_dir_t dir, void *passback, config_data_t *cd,
 
 		/* if error, return error right away */
 		if (result != ISC_R_SUCCESS)
-			return result;
+			return (result);
 	} /* end of while loop */
 
-	return result;
+	return (result);
 }
 
 /*
@@ -611,7 +626,7 @@ fs_allowzonexfr(void *driverarg, void *dbdata, const char *name,
 
  complete_AXFR:
 	isc_mem_free(ns_g_mctx, path);
-	return result;
+	return (result);
 }
 
 static isc_result_t
@@ -679,7 +694,7 @@ fs_allnodes(const char *zone, void *driverarg, void *dbdata,
 	}
 
 	/* process the directory */
-	result = process_dir(dir, allnodes, cd, dir_list, basepathlen);
+	result = process_dir(&dir, allnodes, cd, dir_list, basepathlen);
 
 	/* close the directory */
 	isc_dir_close(&dir);
@@ -704,7 +719,7 @@ fs_allnodes(const char *zone, void *driverarg, void *dbdata,
 		}
 
 		/* process the directory */
-		result = process_dir(dir, allnodes, cd, dir_list, basepathlen);
+		result = process_dir(&dir, allnodes, cd, dir_list, basepathlen);
 
 		/* close the directory */
 		isc_dir_close(&dir);
@@ -730,7 +745,7 @@ fs_allnodes(const char *zone, void *driverarg, void *dbdata,
 	if (basepath != NULL)
 		isc_mem_free(ns_g_mctx, basepath);
 
-	return result;
+	return (result);
 }
 
 static isc_result_t
@@ -769,12 +784,13 @@ fs_findzone(void *driverarg, void *dbdata, const char *name)
  complete_FZ:
 
 	isc_mem_free(ns_g_mctx, path);
-	return result;
+	return (result);
 }
 
 static isc_result_t
 fs_lookup(const char *zone, const char *name, void *driverarg,
-	  void *dbdata, dns_sdlzlookup_t *lookup)
+	  void *dbdata, dns_sdlzlookup_t *lookup,
+	  dns_clientinfomethods_t *methods, dns_clientinfo_t *clientinfo)
 {
 	isc_result_t result;
 	char *path;
@@ -784,6 +800,8 @@ fs_lookup(const char *zone, const char *name, void *driverarg,
 
 	UNUSED(driverarg);
 	UNUSED(lookup);
+	UNUSED(methods);
+	UNUSED(clientinfo);
 
 	if (strcmp(name, "*") == 0)
 		/*
@@ -833,7 +851,7 @@ fs_lookup(const char *zone, const char *name, void *driverarg,
 	}
 
 	/* process any records in the directory */
-	result = process_dir(dir, lookup, (config_data_t *) dbdata, NULL, 0);
+	result = process_dir(&dir, lookup, (config_data_t *) dbdata, NULL, 0);
 
 	/* close the directory */
 	isc_dir_close(&dir);
@@ -841,7 +859,7 @@ fs_lookup(const char *zone, const char *name, void *driverarg,
  complete_lkup:
 
 	isc_mem_free(ns_g_mctx, path);
-	return result;
+	return (result);
 }
 
 static isc_result_t
@@ -937,7 +955,7 @@ fs_create(const char *dlzname, unsigned int argc, char *argv[],
 	*dbdata = cd;
 
 	/* return success */
-	return(ISC_R_SUCCESS);
+	return (ISC_R_SUCCESS);
 
 	/* handle no memory error */
  no_mem:
@@ -996,7 +1014,14 @@ static dns_sdlzmethods_t dlz_fs_methods = {
 	fs_lookup,
 	NULL,
 	fs_allnodes,
-	fs_allowzonexfr
+	fs_allowzonexfr,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
 };
 
 /*%
@@ -1025,7 +1050,7 @@ dlz_fs_init(void)
 		result = ISC_R_UNEXPECTED;
 	}
 
-	return result;
+	return (result);
 }
 
 /*%
